/******************************************************************************
 *   @file   adf4002.c
 *   @brief  Implementation of ADF4002 Driver for NIOS processor.
 *   @author ACostina (adrian.costina@analog.com)
 *
 *******************************************************************************
 * Copyright 2011(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************
 *   SVN Revision: 796
 ******************************************************************************/
#define ADF4002_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "adf4002.h"
#include <altera_avalon_pio_regs.h> // PIO access macros
#include <altera_avalon_spi.h>      // SPI functions
#include <unistd.h>                 // usleep();
#include <system.h>
#if ( USE_ADF4002 == 1 )

/*****************************************************************************/
/* Private variables                                                         */
/*****************************************************************************/
unsigned char   txBuffer[3]     = {0,};     // Tx SPI Buffer.
unsigned char   rxBuffer[3]     = {0,};     // Rx SPI Buffer.

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
volatile int*   adf4002RCounterLatch    = &ADF4002_Data[0].iData;   // Variable to store the latch value to be sent to Reference Counter Latch
volatile int*   adf4002NCounterLatch    = &ADF4002_Data[1].iData;   // Variable to store the latch value to be sent to N Counter Latch
volatile int*   adf4002FunctionLatch    = &ADF4002_Data[2].iData;   // Variable to store the latch value to be sent to Function Latch
volatile int*   adf4002InitLatch        = &ADF4002_Data[3].iData;   // Variable to store the latch value to be sent to Initialization Latch
volatile int*   adf4002InitMethod       = &ADF4002_Data[4].iData;   // Variable used to choose the initialization method
volatile int*   adf4002RCounter         = &ADF4002_Data[5].iData;   // Variable used to set the Reference Counter value
volatile int*   adf4002Antibacklash     = &ADF4002_Data[6].iData;    // Variable used to set the Antibacklash Width
volatile int*   adf4002TestMode         = &ADF4002_Data[7].iData;   // Variable used to set the Test Mode Bits
volatile int*   adf4002LDP              = &ADF4002_Data[8].iData;   // Variable used to set the Lock Detect Precision
volatile int*   adf4002NCounter         = &ADF4002_Data[9].iData;   // Variable used to set the N Counter value
volatile int*   adf4002CPG              = &ADF4002_Data[10].iData;  // Variable used to set the CP gain bit
volatile int*   adf4002FCReset          = &ADF4002_Data[11].iData;  // Variable used to set the F1 Counter reset bit
volatile int*   adf4002FPD1             = &ADF4002_Data[12].iData;  // Variable used to set the Power Down Bit
volatile int*   adf4002FMuxout          = &ADF4002_Data[13].iData;  // Variable used to set the Muxout Control bits
volatile int*   adf4002FPDPolarity      = &ADF4002_Data[14].iData;  // Variable used to set the Phase Detector Polarity bit
volatile int*   adf4002FCPTristate      = &ADF4002_Data[15].iData;  // Variable used to set the CP Three-State bit
volatile int*   adf4002FFBEnable        = &ADF4002_Data[16].iData;  // Variable used to set the Fastlock Enable bits
volatile int*   adf4002FFLMode          = &ADF4002_Data[17].iData;  // Variable used to set the Fastlock Mode bits
volatile int*   adf4002FTCControl       = &ADF4002_Data[18].iData;  // Variable used to set the Timer Counter Control bits
volatile int*   adf4002FCS1             = &ADF4002_Data[19].iData;  // Variable used to set Current Setting 1 bits
volatile int*   adf4002FCS2             = &ADF4002_Data[20].iData;  // Variable used to set Current Setting 2 bits
volatile int*   adf4002FPD2             = &ADF4002_Data[21].iData;  // Variable used to set Power Down 2 bit
volatile int*   adf4002ICReset          = &ADF4002_Data[22].iData;  // Variable used to set the F1 Counter reset bit
volatile int*   adf4002IPD1             = &ADF4002_Data[23].iData;  // Variable used to set the Power Down Bit
volatile int*   adf4002IMuxout          = &ADF4002_Data[24].iData;  // Variable used to set the Muxout Control bits
volatile int*   adf4002IPDPolarity      = &ADF4002_Data[25].iData;  // Variable used to set the Phase Detector Polarity bit
volatile int*   adf4002ICPTristate      = &ADF4002_Data[26].iData;  // Variable used to set the CP Three-State bit
volatile int*   adf4002IFBEnable        = &ADF4002_Data[27].iData;  // Variable used to set the Fastlock Enable bits
volatile int*   adf4002IFLMode          = &ADF4002_Data[28].iData;  // Variable used to set the Fastlock Mode bits
volatile int*   adf4002ITCControl       = &ADF4002_Data[29].iData;  // Variable used to set the Timer Counter Control bits
volatile int*   adf4002ICS1             = &ADF4002_Data[30].iData;  // Variable used to set Current Setting 1 bits
volatile int*   adf4002ICS2             = &ADF4002_Data[31].iData;  // Variable used to set Current Setting 2 bits
volatile int*   adf4002IPD2             = &ADF4002_Data[32].iData;  // Variable used to set Power Down 2 bit
volatile int*   adf4002WriteRLatch      = &ADF4002_Data[33].iData;  // Variable used to activate a write to the Reference Counter Latch
volatile int*   adf4002WriteNLatch      = &ADF4002_Data[34].iData;  // Variable used to activate a write to the N Counter Latch
volatile int*   adf4002WriteFLatch      = &ADF4002_Data[35].iData;  // Variable used to activate a write to the Function Latch
volatile int*   adf4002WriteILatch      = &ADF4002_Data[36].iData;  // Variable used to activate a write to the Initialization Latch
volatile int*   adf4002SPIErr           = &ADF4002_Data[37].iData;  // Variable used to indicate that the command sent on SPI was sent successfully

/******************************************************************************
 * @brief Initialize SPI and the ADF4002 Chip
 *
 * @param None.
 *
 * @return success
 ******************************************************************************/
int ADF4002_Init( void )
{
    // setup SPI interface
    SetCtrl(ADF4002_SPI_CFG);
    IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, (GPIO_LE|GPIO_CE|GPIO_CE2));
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_CE);
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LE);
    if (*adf4002NCounter == 0)
    {
    *adf4002NCounter = 1;       //Make sure N Counter > 0
    }
    if (*adf4002RCounter == 0)
    {
    *adf4002RCounter = 1;       // Make sure Reference Counter > 0
    }
    *adf4002SPIErr   = 0;

    switch (*adf4002InitMethod)
    {
    case 0: ADF4002_InitLatchMethod();
            break;
    case 1: ADF4002_InitCEPinMethod();
            break;
    case 2: ADF4002_InitCounterResetMethod();
            break;
    default: ADF4002_InitLatchMethod();
    }
    return( SUCCESS);
}

/******************************************************************************
 * @brief Initializes the ADF4002 by using the Initialization Latch Method
 *
 * @param None.
 *
 * @return None
 ******************************************************************************/
void ADF4002_InitLatchMethod( void )
{
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_CE);
    *adf4002InitLatch= ( CTRL_INIT_LATCH | ADF4002_I_CR_NORMAL | ADF4002_I_PD1_NORMAL |ADF4002_I_MUXOUT_SDO | ADF4002_I_PDPOL_NEGATIVE | ADF4002_I_CPTS_NORMAL | ADF4002_I_FASTLOCK_DISABLE |ADF4002_I_TCC_3 |ADF4002_I_PD2_NORMAL );
    ADF4002_Set (adf4002InitLatch);
    *adf4002RCounterLatch = ( CTRL_R_COUNTER | ADF4002_R_ABW_2_9n | (*adf4002RCounter << ADF4002_R_COUNTER_POSITION) | ADF4002_R_TMB_NORMAL | ADF4002_R_LDP_5 ) ;
    ADF4002_Set (adf4002RCounterLatch);
    *adf4002NCounterLatch =  ( CTRL_N_COUNTER | (*adf4002NCounter << ADF4002_N_COUNTER_POSITION) | ADF4002_N_CP_SETTING_1 );
    ADF4002_Set (adf4002NCounterLatch);
}

/******************************************************************************
 * @brief Initializes the ADF4002 by using the CE Pin Method
 *
 * @param None.
 *
 * @return None

 ******************************************************************************/
void ADF4002_InitCEPinMethod( void )
{
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_CE);
    *adf4002FunctionLatch = ( CTRL_FUNCTION_LATCH | ADF4002_F_CR_NORMAL | ADF4002_F_PD1_NORMAL |ADF4002_F_MUXOUT_SDO | ADF4002_F_PDPOL_NEGATIVE | ADF4002_F_CPTS_NORMAL | ADF4002_F_FASTLOCK_DISABLE |ADF4002_F_TCC_3 |ADF4002_F_PD2_NORMAL );
    ADF4002_Set (adf4002FunctionLatch);
    *adf4002RCounterLatch = ( CTRL_R_COUNTER | ADF4002_R_ABW_2_9n | (*adf4002RCounter << ADF4002_R_COUNTER_POSITION) | ADF4002_R_TMB_NORMAL | ADF4002_R_LDP_5 ) ;
    ADF4002_Set (adf4002RCounterLatch);
    *adf4002NCounterLatch =  ( CTRL_N_COUNTER | (*adf4002NCounter << ADF4002_N_COUNTER_POSITION) | ADF4002_N_CP_SETTING_1 );
    ADF4002_Set (adf4002NCounterLatch);
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_CE);
}

/******************************************************************************
 * @brief Initializes the ADF4002 by using the Counter Reset Method
 *
 * @param None.
 *
 * @return None
 ******************************************************************************/
void ADF4002_InitCounterResetMethod( void )
{
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_CE);
    *adf4002FunctionLatch = ( CTRL_FUNCTION_LATCH | ADF4002_F_CR_RESET | ADF4002_F_PD1_NORMAL |ADF4002_F_MUXOUT_SDO | ADF4002_F_PDPOL_NEGATIVE | ADF4002_F_CPTS_NORMAL | ADF4002_F_FASTLOCK_DISABLE |ADF4002_F_TCC_3 |ADF4002_F_PD2_NORMAL );
    ADF4002_Set (adf4002FunctionLatch);
    *adf4002RCounterLatch = ( CTRL_R_COUNTER | ADF4002_R_ABW_2_9n | (*adf4002RCounter << ADF4002_R_COUNTER_POSITION) | ADF4002_R_TMB_NORMAL | ADF4002_R_LDP_5 ) ;
    ADF4002_Set (adf4002RCounterLatch);
    *adf4002NCounterLatch =  ( CTRL_N_COUNTER | (*adf4002NCounter << ADF4002_N_COUNTER_POSITION) | ADF4002_N_CP_SETTING_1 );
    ADF4002_Set (adf4002NCounterLatch);
    *adf4002FunctionLatch = ( CTRL_FUNCTION_LATCH | ADF4002_F_CR_NORMAL | ADF4002_F_PD1_NORMAL |ADF4002_F_MUXOUT_SDO | ADF4002_F_PDPOL_NEGATIVE | ADF4002_F_CPTS_NORMAL | ADF4002_F_FASTLOCK_DISABLE |ADF4002_F_TCC_3 |ADF4002_F_PD2_NORMAL );
    ADF4002_Set (adf4002FunctionLatch);
}

/******************************************************************************
 * @brief Send SPI Data
 *
 * @param value.
 *
 * @return SUCCESS if the returned value is the same with the one sent
 *          FAIL otherwise
 ******************************************************************************/
int ADF4002_Set( volatile int *value )
{
    int validation;
    txBuffer[0] = ( unsigned char ) ( ( *value & 0xff0000 ) >> 16 );// Arrange the
    txBuffer[1] = ( unsigned char ) ( ( *value & 0x00ff00 ) >> 8 ); // data to be sent
    txBuffer[2] = ( unsigned char ) ( *value & 0x0000ff );          // in 8 bit packets
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LE);
    alt_avalon_spi_command(ADF4002_SPI, 0, 3, txBuffer, 0, rxBuffer,0);
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_LE);
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LE);
    alt_avalon_spi_command(ADF4002_SPI, 0, 0, txBuffer, 3, rxBuffer,0);
    validation = (int) ( rxBuffer[0] << 16 );
    validation += (int) ( rxBuffer[1] << 8 );
    validation += rxBuffer[2];
    if (validation != *value)
    {
        *adf4002SPIErr = 1;
        return (FAIL);
    }
    return(SUCCESS);
}

/******************************************************************************
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns success
 ******************************************************************************/
int ADF4002_UpdateParameters(void)
{
    usleep (1000);
    *adf4002RCounterLatch   = CTRL_R_COUNTER + (*adf4002RCounter << 2) + (*adf4002Antibacklash << 16)+ (*adf4002TestMode << 18) + (*adf4002LDP << 20);
    *adf4002NCounterLatch   = CTRL_N_COUNTER + (*adf4002NCounter << 8) + (*adf4002CPG << 21);
    *adf4002FunctionLatch   = CTRL_FUNCTION_LATCH + (*adf4002FCReset << 2) + (*adf4002FPD1 <<3 )+ (*adf4002FMuxout << 4) + (*adf4002FPDPolarity << 7) + (*adf4002FCPTristate << 8) + (*adf4002FFBEnable <<9 ) + (*adf4002FFLMode <<10) + (*adf4002FTCControl <<11 ) + (*adf4002FCS1 << 15)+ (* adf4002FCS2 << 18) + (*adf4002FPD2 <<21);
    *adf4002InitLatch       = CTRL_INIT_LATCH + (*adf4002ICReset << 2) + (*adf4002IPD1 <<3 )+ (*adf4002IMuxout << 4) + (*adf4002IPDPolarity << 7) + (*adf4002ICPTristate << 8) + (*adf4002IFBEnable <<9 ) + (*adf4002IFLMode <<10) + (*adf4002ITCControl <<11 ) + (*adf4002ICS1 << 15)+ (* adf4002ICS2 << 18) + (*adf4002IPD2 <<21);

    if(*adf4002WriteRLatch)
    {
        ADF4002_Set (adf4002RCounterLatch);
        *adf4002WriteRLatch = 0;
    }

    if(*adf4002WriteNLatch)
    {
        ADF4002_Set (adf4002NCounterLatch);
        *adf4002WriteNLatch = 0;
    }

    if(*adf4002WriteFLatch)
    {
        ADF4002_Set (adf4002FunctionLatch);
        *adf4002WriteFLatch = 0;
    }

    if(*adf4002WriteILatch)
    {
        ADF4002_Set (adf4002InitLatch);
        *adf4002WriteILatch = 0;
    }

    return(SUCCESS);
}
#endif
